<?php

namespace App\Http\Controllers;

use App\Mail\ForgotPasswordCodeMail;
use App\Mail\TestEmail;
use App\Models\Department;
use App\Models\DocType;
use App\Models\DocuemntEntry;
use App\Models\Status;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use League\CommonMark\Node\Block\Document;
use Psy\Util\Docblock;
use Illuminate\Support\Facades\Mail;


class DocumentController extends Controller
{
    /**
     * Display a listing of the resource.
     */




    public function index(Request $request)
    {

        $query = DocuemntEntry::select("docuemnt_entries.*","departments.dep_name as dept_name","doc_types.doc_type","users.name","users.phone as phone","status.status")
        ->join('departments','departments.id','docuemnt_entries.department_id')
        ->join('doc_types','doc_types.id','docuemnt_entries.doctypeid')
        ->join('users','users.id','docuemnt_entries.emp_id')
        ->join('status','status.id','docuemnt_entries.status_id');

         //status_id
        if($request->dept_id){
            $query->where('docuemnt_entries.department_id' ,$request->dept_id)  ;
        }

        //status_id
        if($request->status_id){
            $query->where('docuemnt_entries.status_id' ,$request->status_id)  ;
        }

        if(auth()->user()->role_id == 5) {
            $query->where('emp_id' ,auth()->user()->id);
        }
        

        if($request->search_key){
          //  dd($request->search_key);
            $query->where('departments.dep_name' , 'like', '%'.$request->search_key.'%' )  ;
        }
        $query->orderBy('docuemnt_entries.id','desc');
        $query->limit(100);


        $documents = $query->get();
        
        $departments = Department::get();
        $status = Status::get();
        

        return view('backend/document/list',compact('documents','departments','status', 'request'));

    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {

        // $randomgen = 'D'.time().rand(10,1000).'COU';
        // $randomgen = time().rand(10,1000);
        $randomgen = 'D'.time().rand(10,100);

        $departments = Department::get();
        $documents = DocType::get();

    
        return view('backend/document/create',compact('departments','documents','randomgen'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'tracking_id' => 'required',
            'received_from' => 'required',
            'doctypeid' => 'required',
            'emp_id' => 'required',
           // 'entry_by' => 'required',

          ]);

          
          $data['tracking_id'] = $request->tracking_id;
          $data['received_from'] = $request->received_from;
          $data['doctypeid'] = $request->doctypeid;
          $data['department_id'] = $request->department_id;
          $data['emp_id'] = $request->emp_id;
          $data['entry_by'] = Auth::user()->id;
          $data['status'] = "Received";
          $data['status_id'] = 1;
          $data['note'] = $request->note;


        
          $departments = Department::get();

      
          DocuemntEntry::insert($data);
      
          $dtype = DocType::where('id',$request->doctypeid)->first(); 
         
          $user = User::where('id',$request->emp_id)->first(); 

        
          Mail::to($user->email)->send(new TestEmail);
        //  Mail::to($user->email)->send(new ForgotPasswordCodeMail($user, $request->received_from, $expiresAt));

         sendSms($user->phone,'Received a '.$dtype->doc_type.' from '.$request->received_from.'. Tracking ID : '.$request->tracking_id).'COU,Admin';
         

          // Sending directly without a mailable class:
        //   Mail::raw('This is a plain text email', function ($message) {
        //       $message->to('rinqil@gmail.com');
        //       $message->subject('Test Email from Laravel');
        //   });


          return redirect()->back()->with('success', 'Received done.');

          
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        return view('backend/document/show');
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        return view('backend/document/edit');
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
       //
    }

    public function  change_status(Request $request){ 
        $data  = DocuemntEntry::where('id',$request->id)->update(['status_id'=> $request->status ]);
        return $data ;

    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        //
    }

    public function docType()
    {

        return view('backend/document/create_doc_type');
       
    }

    public function storeDocType(Request $request) {

        $this->validate($request, [
           // 'id' => 'required',
            'doc_type' => 'required',
                
          ]);
       //   $data['id'] = $request->id;
          $data['doc_type'] = $request->doc_type;
      
          DocType::insert($data);
      
          return redirect()->back()->with('success', 'Document type input successfully.');
          //dd($request->all());
    }


    public function dList()
    {


        $documents = DocuemntEntry::select("docuemnt_entries.*","departments.dep_name as dept_name","doc_types.doc_type","users.name","users.phone as phone")
        ->join('departments','departments.id','docuemnt_entries.department_id')
        ->join('doc_types','doc_types.id','docuemnt_entries.doctypeid')
        ->join('users','users.id','docuemnt_entries.emp_id')
        ->join('users','users.id','docuemnt_entries.emp_id')
        ->get();


       $documents = DocuemntEntry:: get();

        return $documents;

    }


    public function dTypeList()
    {



       $docTypeList = DocType:: get();

       return view('backend/document/doctypelist', compact('docTypeList'));

    }


    public function typeDelete(string $id)
    {
        DocType::destroy($id);
        return redirect()->back()->with('success', 'Delete done.');
  
    }

}
