<?php

namespace App\Http\Controllers;

use App\Mail\ForgotPasswordCodeMail;
use App\Models\Department;
use App\Models\Role;
use App\Models\User;
use Illuminate\Http\Request;
use DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;

class UserController extends Controller
{
  /**
   * Display a listing of the resource.
   */
  public function index()
  {


    $users = User::select("users.*", "departments.dep_name as dept_name", "role.role_name")
      ->join('departments', 'departments.id', 'users.dep_id')
      ->join('role', 'role.id', 'users.role_id')
      ->get();

    return view('backend/user/list', ['users' => $users]);
  }

  public function usersApi(Request $request)
  {


    $users = User::select("users.*", "departments.dep_name as dept_name")
      ->join('departments', 'departments.id', 'users.dep_id')
      ->where('users.dep_id', $request->dep_id)
      ->get();

    return $users;
  }

  /**
   * Show the form for creating a new resource.
   */
  public function create()
  {

    //  $departments = DB::select('select * from departments');

    $departments = Department::get();
    $roles = Role::get();

    return view('backend/user/create', compact('departments', 'roles'));
  }

  /**
   * Store a newly created resource in storage.
   */
  public function store(Request $request)
  {
    $this->validate($request, [
      'userName' => 'required|min:5',
      'email' => 'required|email|unique:users',
      'departmentid' => 'required',
      'role_id' => 'required',
      'phone' => 'required|max:11|min:9',
      'password' => 'required|min:5|confirmed',

    ]);
    $data['name'] = $request->userName;
    $data['email'] = $request->email;
    $data['dep_id'] = $request->departmentid;
    $data['role_id'] = $request->role_id;
    $data['phone'] = $request->phone;
    $data['password'] = Hash::make($request->password);

    User::insert($data);

    return redirect()->back()->with('success', 'Registration done.');
    //dd($request->all());
  }

  /**
   * Display the specified resource.
   */
  public function show(string $id)
  {
    return view('backend/user/show');
  }
  /**
   * Show the form for editing the specified resource.
   */
  public function edit(string $id)
  {

    $user = User::find($id);

    return view('backend/user/edit', ['user' => $user]);
  }

  /**
   * Update the specified resource in storage.
   */
  public function update(Request $request, string $id)
  {
    $this->validate($request, [
      'userName' => 'required|min:5',
      'email' => 'required|email|unique:users,email, ' . $id,
      // 'departmentid' => 'required',
      // 'password' => 'required|min:5|confirmed',

    ]);
    //1st
    $user = User::find($id);
    $user->name = $request->userName;
    $user->email = $request->email;
    $user->save();

    //   dd($user);

    //   exit();



    // 2nd
    //   $data['name'] = $request->name;
    //   $data['email'] = $request->email;
    //   User::where('id',$id)->update($data);

    return redirect()->back()->with('success', 'Edit done.');
  }

  /**
   * Remove the specified resource from storage.
   */
  public function destroy(string $id)
  {
      User::destroy($id);
      return redirect()->back()->with('success', 'Delete done.');

  }

  public function logout(Request $request)
  {
    Auth::logout();
    return redirect()->to('/');
    // return view('/welcome');
  }


  public function changePassword()
  {
    return view('backend/settings/change_password');
  }


  public function changePasswordSubmit(Request $request)
  {

    $user = User::find(Auth::user()->id);
    if ($request->newpassword != $request->confirmpassword) {

      return redirect()->back()->with('error', 'New Password and confirm password not matched.');
    }

    if (!Hash::check($request->currentpassword, $user->password)) {

      return redirect()->back()->with('error', 'Current Password not matched.');
    }

    $user->password = Hash::make($request->newpassword);
    $user->save();


    return redirect()->back()->with('success', 'Password Changed successfully.');
  }


  public function forgotpasswordmail()
  {
    return view('backend/settings/forgot_password_email');
  }

  public function codeSendmail(Request $request)
  {
    $this->validate($request, [
      'email' => 'required|email|exists:users,email',
    ]);

    $user = User::where('email', $request->email)->first();

    $forgotPasswordCode = rand(100000, 999999);
    $expiresAt = now()->addMinutes(30);
    // echo $expiresAt;
    // die;
    $user->reset_token = $forgotPasswordCode;
    $user->reset_token_expare = $expiresAt;
    $user->save();

    Mail::to($user->email)->send(new ForgotPasswordCodeMail($user, $forgotPasswordCode, $expiresAt));
    // return $this->sendResponse('', 'We have mailed your reset code!');
    return redirect()->to('/forgotpasswordtoken?email='.$user->email)->with('success', 'We have mailed your reset code!');
  }

  public function forgotpasswordtoken(Request $request)
  {
    return view('backend/settings/forgot_password_token')->with('request',$request);
  }

  public function forgot_password_code(Request $request)
  {
    // $this->validate($request, [
    //     'password' => 'required|confirmed|min:6',
    // ]);
      // dd($request->all());
    if (strlen($request->verification_code) <= 4) {
      // User not found
      return $this->sendError('Verification Code is required');
    }
    if (strlen($request->email) <= 4) {
      // User not found
      return $this->sendError('Email is required');
    }
    $email = $request->email;
    $verificationCode = $request->verification_code;
    $user = User::where('email', $email)->first();

    if (!$user) {
      // User not found
      return redirect()->back()->with('error', 'The email is not registered.');
    }

    if ($user->reset_token !== $verificationCode) {
      // Incorrect verification code
      return redirect()->back()->with('error', 'The verification code is incorrect.');
    }

    // dd($user->verification_code_expires_at);
    if ($user->reset_token_expare < now()) {
      // Verification code has expired
      return redirect()->back()->with('error', 'The verification code has expired.');
    }

    return redirect()->to('/forgotpasswordreset?email='.$user->email.'&verification_code='.$verificationCode)->with('success', 'Code is verified.');

  }

  public function forgotpasswordreset(Request $request)
  {
    return view('backend/settings/forgot_password_reset')->with('request',$request);
  }
  public function resetPassword(Request $request)
  {
    if (strlen($request->password) <= 5) {
      // User not found
      return redirect()->back()->with('error', 'The password must be at least 5 characters');
    }
    if ($request->password != $request->confirm_password) {
      // User not found
      return redirect()->back()->with('error', 'The password confirmation does not match');
    }
    if (strlen($request->verification_code) <= 4) {
      // User not found
      return redirect()->back()->with('error', 'Verification Code is required');
    }
    $email = $request->email;
    $verificationCode = $request->verification_code;
    $user = User::where('email', $email)->first();

    if (!$user) {
      // User not found
      return redirect()->back()->with('error', 'The email is not registered.');
    }

    if ($user->reset_token !== $verificationCode) {
      // Incorrect verification code
      return redirect()->back()->with('error', 'The verification code is incorrect.');
      
    }

    // dd($user->verification_code_expires_at);
    if ($user->reset_token_expare < now()) {
      // Verification code has expired
      return redirect()->back()->with('error', 'The verification code has expired.');
    }

    $user->password = Hash::make($request->password);
    $user->reset_token = null;
    $user->reset_token_expare = null;
    $user->save();
    return redirect()->to('/signin')->with('success', 'Your password has been reset!');
  }
}
